# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from .item_request import ItemRequest
from ..request_builder_base import RequestBuilderBase
from ..request.item_create_session import ItemCreateSessionRequestBuilder
from ..request.item_copy import ItemCopyRequestBuilder
from ..request.item_create_link import ItemCreateLinkRequestBuilder
from ..request.item_invite import ItemInviteRequestBuilder
from ..request.item_delta import ItemDeltaRequestBuilder
from ..request.item_search import ItemSearchRequestBuilder
from ..request.item_content_request import ItemContentRequestBuilder


class ItemRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client):
        """Initialize the ItemRequestBuilder

        Args:
            request_url (str): The url to perform the ItemRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
        """
        super(ItemRequestBuilder, self).__init__(request_url, client)

    def request(self, expand=None, select=None, options=None):
        """Builds the ItemRequest

        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request. Defaults to None.

        Returns:
            :class:`ItemRequest<onedrivesdk.request.item_request.ItemRequest>`:
                The ItemRequest
        """
        req = ItemRequest(self._request_url, self._client, options)
        req._set_query_options(expand=expand, select=select)
        return req

    def delete(self):
        """Deletes the specified Item."""
        self.request().delete()

    def get(self):
        """Gets the specified Item.
        
        Returns:
            :class:`Item<onedrivesdk.model.item.Item>`:
                The Item.
        """
        return self.request().get()

    def update(self, item):
        """Updates the specified Item.
        
        Args:
            item (:class:`Item<onedrivesdk.model.item.Item>`):
                The Item to update.

        Returns:
            :class:`Item<onedrivesdk.model.item.Item>`:
                The updated Item
        """
        return self.request().update(item)


    def upload(self, local_path):
        """Uploads the file using PUT
        
        Args:
            local_path (str): The path to the local file to upload.

        Returns: 
            The created entity.
        """
        return self.content.request().upload(local_path)


    def download(self, local_path):
        """Downloads the specified entity.

        Args:
            local_path (str): The path where the entity should be
                downloaded to
        """
        return self.content.request().download(local_path)


    @property
    def permissions(self):
        """The permissions for the ItemRequestBuilder

        Returns: 
            :class:`PermissionsCollectionRequestBuilder<onedrivesdk.request.permissions_collection.PermissionsCollectionRequestBuilder>`:
                A request builder created from the ItemRequestBuilder
        """
        return PermissionsCollectionRequestBuilder(self.append_to_request_url("permissions"), self._client)

    @property
    def subscriptions(self):
        """The subscriptions for the ItemRequestBuilder

        Returns: 
            :class:`SubscriptionsCollectionRequestBuilder<onedrivesdk.request.subscriptions_collection.SubscriptionsCollectionRequestBuilder>`:
                A request builder created from the ItemRequestBuilder
        """
        return SubscriptionsCollectionRequestBuilder(self.append_to_request_url("subscriptions"), self._client)

    @property
    def versions(self):
        """The versions for the ItemRequestBuilder

        Returns: 
            :class:`VersionsCollectionRequestBuilder<onedrivesdk.request.versions_collection.VersionsCollectionRequestBuilder>`:
                A request builder created from the ItemRequestBuilder
        """
        return VersionsCollectionRequestBuilder(self.append_to_request_url("versions"), self._client)

    @property
    def children(self):
        """The children for the ItemRequestBuilder

        Returns: 
            :class:`ChildrenCollectionRequestBuilder<onedrivesdk.request.children_collection.ChildrenCollectionRequestBuilder>`:
                A request builder created from the ItemRequestBuilder
        """
        return ChildrenCollectionRequestBuilder(self.append_to_request_url("children"), self._client)

    @property
    def tags(self):
        """The tags for the ItemRequestBuilder

        Returns: 
            :class:`TagsCollectionRequestBuilder<onedrivesdk.request.tags_collection.TagsCollectionRequestBuilder>`:
                A request builder created from the ItemRequestBuilder
        """
        return TagsCollectionRequestBuilder(self.append_to_request_url("tags"), self._client)

    @property
    def thumbnails(self):
        """The thumbnails for the ItemRequestBuilder

        Returns: 
            :class:`ThumbnailsCollectionRequestBuilder<onedrivesdk.request.thumbnails_collection.ThumbnailsCollectionRequestBuilder>`:
                A request builder created from the ItemRequestBuilder
        """
        return ThumbnailsCollectionRequestBuilder(self.append_to_request_url("thumbnails"), self._client)

    @property
    def content(self):
        """The content for the ItemRequestBuilder

        Returns: 
            :class:`ItemContentRequestBuilder<onedrivesdk.request.item_content.ItemContentRequestBuilder>`:
                A request builder created from the ItemRequestBuilder
        """
        return ItemContentRequestBuilder(self.append_to_request_url("content"), self._client)
    def create_session(self, item=None):
        """Executes the createSession method

        Args:
            item (:class:`ChunkedUploadSessionDescriptor<onedrivesdk.model.chunked_upload_session_descriptor.ChunkedUploadSessionDescriptor>`):
                Defaults to None, The item to use in the method request

        Returns:
            :class:`ItemCreateSessionRequestBuilder<onedrivesdk.request.item_create_session.ItemCreateSessionRequestBuilder>`:
                A ItemCreateSessionRequestBuilder for the method
        """
        return ItemCreateSessionRequestBuilder(self.append_to_request_url("upload.createSession"), self._client, item=item)

    def copy(self, name=None, parent_reference=None):
        """Executes the copy method

        Args:
            name (str):
                The name to use in the method request          
            parent_reference (:class:`ItemReference<onedrivesdk.model.item_reference.ItemReference>`):
                Defaults to None, The parent_reference to use in the method request

        Returns:
            :class:`ItemCopyRequestBuilder<onedrivesdk.request.item_copy.ItemCopyRequestBuilder>`:
                A ItemCopyRequestBuilder for the method
        """
        return ItemCopyRequestBuilder(self.append_to_request_url("action.copy"), self._client, name=name, parent_reference=parent_reference)

    def create_link(self, type):
        """Executes the createLink method

        Args:
            type (str):
                The type to use in the method request          

        Returns:
            :class:`ItemCreateLinkRequestBuilder<onedrivesdk.request.item_create_link.ItemCreateLinkRequestBuilder>`:
                A ItemCreateLinkRequestBuilder for the method
        """
        return ItemCreateLinkRequestBuilder(self.append_to_request_url("action.createLink"), self._client, type)

    def invite(self, recipients, require_sign_in=None, roles=None, send_invitation=None, message=None):
        """Executes the invite method

        Args:
            require_sign_in (bool):
                The require_sign_in to use in the method request          
            roles (str):
                The roles to use in the method request          
            recipients (:class:`Recipients<onedrivesdk.model.recipients.Recipients>`):
                The recipients to use in the method request
            send_invitation (bool):
                The send_invitation to use in the method request          
            message (str):
                The message to use in the method request          

        Returns:
            :class:`ItemInviteRequestBuilder<onedrivesdk.request.item_invite.ItemInviteRequestBuilder>`:
                A ItemInviteRequestBuilder for the method
        """
        return ItemInviteRequestBuilder(self.append_to_request_url("action.invite"), self._client, recipients, require_sign_in=require_sign_in, roles=roles, send_invitation=send_invitation, message=message)

    def delta(self, token=None):
        """Executes the delta method

        Args:
            token (str):
                The token to use in the method request          

        Returns:
            :class:`ItemDeltaRequestBuilder<onedrivesdk.request.item_delta.ItemDeltaRequestBuilder>`:
                A ItemDeltaRequestBuilder for the method
        """
        return ItemDeltaRequestBuilder(self.append_to_request_url("view.delta"), self._client, token=token)

    def search(self, q=None):
        """Executes the search method

        Args:
            q (str):
                The q to use in the method request          

        Returns:
            :class:`ItemSearchRequestBuilder<onedrivesdk.request.item_search.ItemSearchRequestBuilder>`:
                A ItemSearchRequestBuilder for the method
        """
        return ItemSearchRequestBuilder(self.append_to_request_url("view.search"), self._client, q=q)

from ..request.permissions_collection import PermissionsCollectionRequestBuilder
from ..request.subscriptions_collection import SubscriptionsCollectionRequestBuilder
from ..request.versions_collection import VersionsCollectionRequestBuilder
from ..request.children_collection import ChildrenCollectionRequestBuilder
from ..request.tags_collection import TagsCollectionRequestBuilder
from ..request.thumbnails_collection import ThumbnailsCollectionRequestBuilder



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..one_drive_object_base import OneDriveObjectBase


class Thumbnail(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def height(self):
        """Gets and sets the height
        
        Returns: 
            int:
                The height
        """
        if "height" in self._prop_dict:
            return self._prop_dict["height"]
        else:
            return None

    @height.setter
    def height(self, val):
        self._prop_dict["height"] = val

    @property
    def url(self):
        """Gets and sets the url
        
        Returns: 
            str:
                The url
        """
        if "url" in self._prop_dict:
            return self._prop_dict["url"]
        else:
            return None

    @url.setter
    def url(self, val):
        self._prop_dict["url"] = val

    @property
    def width(self):
        """Gets and sets the width
        
        Returns: 
            int:
                The width
        """
        if "width" in self._prop_dict:
            return self._prop_dict["width"]
        else:
            return None

    @width.setter
    def width(self, val):
        self._prop_dict["width"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.storage_plans import StoragePlans
from ..one_drive_object_base import OneDriveObjectBase


class Quota(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def deleted(self):
        """Gets and sets the deleted
        
        Returns: 
            int:
                The deleted
        """
        if "deleted" in self._prop_dict:
            return self._prop_dict["deleted"]
        else:
            return None

    @deleted.setter
    def deleted(self, val):
        self._prop_dict["deleted"] = val

    @property
    def remaining(self):
        """Gets and sets the remaining
        
        Returns: 
            int:
                The remaining
        """
        if "remaining" in self._prop_dict:
            return self._prop_dict["remaining"]
        else:
            return None

    @remaining.setter
    def remaining(self, val):
        self._prop_dict["remaining"] = val

    @property
    def state(self):
        """Gets and sets the state
        
        Returns: 
            str:
                The state
        """
        if "state" in self._prop_dict:
            return self._prop_dict["state"]
        else:
            return None

    @state.setter
    def state(self, val):
        self._prop_dict["state"] = val

    @property
    def total(self):
        """Gets and sets the total
        
        Returns: 
            int:
                The total
        """
        if "total" in self._prop_dict:
            return self._prop_dict["total"]
        else:
            return None

    @total.setter
    def total(self, val):
        self._prop_dict["total"] = val

    @property
    def used(self):
        """Gets and sets the used
        
        Returns: 
            int:
                The used
        """
        if "used" in self._prop_dict:
            return self._prop_dict["used"]
        else:
            return None

    @used.setter
    def used(self, val):
        self._prop_dict["used"] = val

    @property
    def storage_plans(self):
        """
        Gets and sets the storagePlans
        
        Returns: 
            :class:`StoragePlans<onedrivesdk.model.storage_plans.StoragePlans>`:
                The storagePlans
        """
        if "storagePlans" in self._prop_dict:
            if isinstance(self._prop_dict["storagePlans"], OneDriveObjectBase):
                return self._prop_dict["storagePlans"]
            else :
                self._prop_dict["storagePlans"] = StoragePlans(self._prop_dict["storagePlans"])
                return self._prop_dict["storagePlans"]

        return None

    @storage_plans.setter
    def storage_plans(self, val):
        self._prop_dict["storagePlans"] = val



'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
from ..request.one_drive_client import OneDriveClient
from ..request.drive_request_builder import DriveRequestBuilder


def item(self, drive=None, id=None, path=None):
    """Gets an item given the specified (optional) drive,
    (optional) id, and (optional) path

    Args:
        drive (str): The drive that you want to use
        id (str): The id of the item to request
        path (str): The path of the item to request

    Returns:
        :class:`ItemRequestBuilder<onedrivesdk.requests.item_request_builder.ItemRequestBuilder>`:
            A request builder for an item given a path or id
    """
    if id is None and path is None:
        raise ValueError("Either 'path' or 'id' must be specified")
    elif id is not None and path is not None:
        raise ValueError("Only one of either 'path' or 'id' can be specified")

    drive_builder = self.drives[drive] if drive else self.drive

    if path:
        return drive_builder.item_by_path(path)
    elif id:
        return drive_builder.items[id]


@property
def drive(self):
    """Gets the user's default drive

    Returns: 
        :class:`DriveRequestBuilder<onedrivesdk.requests.drive_request_builder.DriveRequestBuilder>`:
            User's default drive
    """
    return DriveRequestBuilder(self.base_url + "drive", self)

OneDriveClient.item = item
OneDriveClient.drive = drive



# -*- coding: utf-8 -*- 
'''
# Copyright (c) 2015 Microsoft Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..one_drive_object_base import OneDriveObjectBase


class AsyncOperationStatus(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def operation(self):
        """Gets and sets the operation
        
        Returns: 
            str:
                The operation
        """
        if "operation" in self._prop_dict:
            return self._prop_dict["operation"]
        else:
            return None

    @operation.setter
    def operation(self, val):
        self._prop_dict["operation"] = val

    @property
    def percentage_complete(self):
        """Gets and sets the percentageComplete
        
        Returns: 
            float:
                The percentageComplete
        """
        if "percentageComplete" in self._prop_dict:
            return self._prop_dict["percentageComplete"]
        else:
            return None

    @percentage_complete.setter
    def percentage_complete(self, val):
        self._prop_dict["percentageComplete"] = val

    @property
    def status(self):
        """Gets and sets the status
        
        Returns: 
            str:
                The status
        """
        if "status" in self._prop_dict:
            return self._prop_dict["status"]
        else:
            return None

    @status.setter
    def status(self, val):
        self._prop_dict["status"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) 2015 Microsoft Corporation
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from .permission_request import PermissionRequest
from ..request_builder_base import RequestBuilderBase
import asyncio


class PermissionRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client):
        """Initialize the PermissionRequestBuilder

        Args:
            request_url (str): The url to perform the PermissionRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
        """
        super(PermissionRequestBuilder, self).__init__(request_url, client)

    def request(self, expand=None, select=None, options=None):
        """Builds the PermissionRequest

        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request. Defaults to None.

        Returns:
            :class:`PermissionRequest<onedrivesdk.request.permission_request.PermissionRequest>`:
                The PermissionRequest
        """
        req = PermissionRequest(self._request_url, self._client, options)
        req._set_query_options(expand=expand, select=select)
        return req

    def delete(self):
        """Deletes the specified Permission."""
        self.request().delete()

    @asyncio.coroutine
    def delete_async(self):
        """Deletes the specified Permission."""
        yield from self.request().delete_async()
    def get(self):
        """Gets the specified Permission.
        
        Returns:
            :class:`Permission<onedrivesdk.model.permission.Permission>`:
                The Permission.
        """
        return self.request().get()

    @asyncio.coroutine
    def get_async(self):
        """Gets the specified Permission in async.

        Returns:
            :class:`Permission<onedrivesdk.model.permission.Permission>`:
                The Permission.
        """
        entity = yield from self.request().get_async()
        return entity
    def update(self, permission):
        """Updates the specified Permission.
        
        Args:
            permission (:class:`Permission<onedrivesdk.model.permission.Permission>`):
                The Permission to update.

        Returns:
            :class:`Permission<onedrivesdk.model.permission.Permission>`:
                The updated Permission
        """
        return self.request().update(permission)

    @asyncio.coroutine
    def update_async(self, permission):
        """Updates the specified Permission in async
        
        Args:
            permission (:class:`Permission<onedrivesdk.model.permission.Permission>`):
                The Permission to update.

        Returns:
            :class:`Permission<onedrivesdk.model.permission.Permission>`:
                The updated Permission.
        """
        entity = yield from self.request().update_async(permission)
        return entity



'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''


class RequestBuilderBase(object):

    def __init__(self, request_url, client):
        """Initialize a request builder which returns a request
        when request() is called

        Args:
            request_url (str): The URL to construct the request
                for
            client (:class:`OneDriveClient<onedrivesdk.requests.one_drive_client.OneDriveClient>`):
                The client with which the request will be made
        """
        self._request_url = request_url
        self._client = client

    def append_to_request_url(self, url_segment):
        """Appends a URL portion to the current request URL

        Args:
            url_segment (str): The segment you would like to append
                to the existing request URL.
        """
        return self._request_url + "/" + url_segment



'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
from __future__ import generators, unicode_literals
import json
from .error import OneDriveError


class HttpResponse(object):

    def __init__(self, status, headers, content):
        """Initialize the HttpResponse class returned after
        an HTTP request is made

        Args:
            status (int): HTTP status (ex. 200, 201, etc.)
            headers (dict of (str, str)): The headers in the
                response
            content (str): The body of the response
        """
        self._status = status
        self._headers = headers
        self._content = content

        if self.content and (self.status < 200 or self.status >= 300):
            try:
                message = json.loads(self.content)
            except ValueError:  # Invalid or empty response message
                from .error import ErrorCode
                message = {
                    "error": {
                        "code": ErrorCode.Malformed,
                        "message": "The following invalid JSON was returned:\n%s" % self.content
                        }
                    }

            if "error" in message:
                if type(message["error"]) == dict:
                    raise OneDriveError(message["error"], self.status)
                else:
                    raise Exception(str(message["error"]))

    def __str__(self):
        properties = {
            'Status': self.status,
            'Headers': self.headers,
            'Content': self.content
            }
        ret = ""
        for k, v in properties.items():
            ret += "{}: {}\n".format(k, v)
        return ret

    @property
    def status(self):
        """The HTTP status of the response

        Returns:
            int: HTTP status
        """
        return self._status

    @property
    def headers(self):
        """The headers of the response

        Returns:
            dict of (str, str):
                The headers used by the response
        """
        return self._headers

    @property
    def content(self):
        """The content of the response

        Returns:
            str: The body of the response
        """
        return self._content



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from ..model.permission import Permission
from ..collection_base import CollectionRequestBase
from ..request_builder_base import RequestBuilderBase
from ..options import *
import json
import asyncio


class ItemInviteRequest(CollectionRequestBase):

    def __init__(self, request_url, client, options, recipients, require_sign_in=None, roles=None, send_invitation=None, message=None):
        super(ItemInviteRequest, self).__init__(request_url, client, options)
        self.method = "POST"
        self.body_options={}

        if recipients:
            self.body_options["recipients"] = recipients
        if require_sign_in:
            self.body_options["requireSignIn"] = require_sign_in
        if roles:
            self.body_options["roles"] = roles
        if send_invitation:
            self.body_options["sendInvitation"] = send_invitation
        if message:
            self.body_options["message"] = message

    @property
    def body_options(self):
        return self._body_options

    @body_options.setter
    def body_options(self, value):
        self._body_options=value

    def post(self):
        """Sends the POST request
        
        Returns: 
            :class:`ItemsCollectionResponse<onedrivesdk.request.items_collection.ItemsCollectionResponse>`:
                The resulting collection page from the operation
        """
        self.content_type = "application/json"
        collection_response = ItemsCollectionResponse(json.loads(self.send(self.body_options).content))
        return self._page_from_response(collection_response)

    @asyncio.coroutine
    def post_async(self):
        """Sends the POST request using an asyncio coroutine

        Yields:
            :class:`ItemsCollectionResponse<onedrivesdk.request.items_collection.ItemsCollectionResponse>`:
                The resulting collection page from the operation
        """
        future = self._client._loop.run_in_executor(None,
                                                    self.post)
        collection_response = yield from future
        return collection_response


class ItemInviteRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client, recipients, require_sign_in=None, roles=None, send_invitation=None, message=None):
        super(ItemInviteRequestBuilder, self).__init__(request_url, client)
        self._method_options = {}

        self._method_options["recipients"] = recipients._prop_dict
        self._method_options["requireSignIn"] = require_sign_in
        self._method_options["roles"] = roles
        self._method_options["sendInvitation"] = send_invitation
        self._method_options["message"] = message

    def request(self, expand=None, select=None, top=None, order_by=None, options=None):
        """Builds the request for the ItemInvite
        
        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            top (int): Default None, the number of items to return in a result.
            order_by (str): Default None, comma-seperated list of properties
                that are used to sort the order of items in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                Default to None, list of options to include in the request

        Returns: 
            :class:`ItemInviteRequest<onedrivesdk.request.item_invite.ItemInviteRequest>`:
                The request
        """
        req = ItemInviteRequest(self._request_url, self._client, options, self._method_options["recipients"], require_sign_in=self._method_options["requireSignIn"], roles=self._method_options["roles"], send_invitation=self._method_options["sendInvitation"], message=self._method_options["message"])
        req._set_query_options(expand=expand, select=select, top=top, order_by=order_by)
        return req

    def post(self):
        """Sends the POST request
        
        Returns:
            :class:`ItemsCollectionResponse<onedrivesdk.request.items_collection.ItemsCollectionResponse>`:
            The resulting ItemsCollectionResponse from the operation
        """
        return self.request().post()

    @asyncio.coroutine
    def post_async(self):
        """Sends the POST request using an asyncio coroutine
        
        Yields:
            :class:`ItemsCollectionResponse<onedrivesdk.request.items_collection.ItemsCollectionResponse>`:
                The resulting ItemsCollectionResponse from the operation
        """
        collection_page = yield from self.request().post_async()
        return collection_page

from ..request.items_collection import ItemsCollectionResponse



'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
try:
    from http.server import HTTPServer, BaseHTTPRequestHandler
except ImportError:
    from SimpleHTTPServer import SimpleHTTPRequestHandler as BaseHTTPRequestHandler
    from SocketServer import TCPServer as HTTPServer

try:
    from urllib.parse import urlparse, parse_qs, unquote
except ImportError:
    from urlparse import urlparse, parse_qs
    from urllib import unquote

import threading
import webbrowser

def get_auth_code(auth_url, redirect_uri):
    """Easy way to get the auth code. Wraps up all the threading
    and stuff. Does block main thread.

    Args:
        auth_url (str): URL of auth server.
        redirect_uri (str): Redirect URI, as set for the app. Should be 
            something like "http://localhost:8080" for this to work.

    Returns: 
        str: A string representing the auth code, sent back by the server
    """
    url_netloc = urlparse(redirect_uri).netloc
    if ':' not in url_netloc:
        host_address = url_netloc
        port = 80 # default port
    else:
        host_address, port = url_netloc.split(':')
        port = int(port)
    # Set up HTTP server and thread
    code_acquired = threading.Event()
    s = GetAuthCodeServer((host_address, port), code_acquired, GetAuthCodeRequestHandler)    
    th = threading.Thread(target=s.serve_forever)
    th.start()
    webbrowser.open(auth_url)
    # At this point the browser will open and the code
    # will be extracted by the server
    code_acquired.wait()  # First wait for the response from the auth server
    code = s.auth_code
    s.shutdown()
    th.join()
    return code


class GetAuthCodeServer(HTTPServer, object):

    def __init__(self, server_address, stop_event, RequestHandlerClass):
        HTTPServer.__init__(self, server_address, RequestHandlerClass)
        self._stop_event = stop_event
        self.auth_code = None

    @property
    def auth_code(self):
        return self._auth_code

    @auth_code.setter
    def auth_code(self, value):
        self._auth_code = value
        if value is not None:
            self._stop_event.set()


class GetAuthCodeRequestHandler(BaseHTTPRequestHandler):

    def do_GET(self):
        params = parse_qs(urlparse(self.path).query)
        if "code" in params:
            # Extract the code query param
            self.server.auth_code = params["code"][0]
        if "error" in params:
            error_msg, error_desc = (unquote(params["error"][0]),
                                     unquote(params["error_description"][0]))
            raise RuntimeError("The server returned an error: {} - {}"
                               .format(error_msg, error_desc))
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()
        self.wfile.write(bytes(
            '<script type="text/javascript">window.close()</script>'
            .encode("utf-8")))



'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
from __future__ import unicode_literals
import json
from .auth_provider_base import AuthProviderBase
from .options import *
from .session import Session
import sys

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode


class AuthProvider(AuthProviderBase):

    MSA_AUTH_SERVER_URL = "https://login.live.com/oauth20_authorize.srf"
    MSA_AUTH_TOKEN_URL = "https://login.live.com/oauth20_token.srf"

    def __init__(self, http_provider, client_id=None, scopes=None, access_token=None, session_type=None, loop=None,
                 auth_server_url=None, auth_token_url=None):
        """Initialize the authentication provider for authenticating
        requests sent to OneDrive

        Args:
            http_provider (:class:`HttpProviderBase<onedrivesdk.http_provider_base>`):
                The HTTP provider to use for all auth requests
            client_id (str): Defaults to None, the client id for your
                application
            scopes (list of str): Defaults to None, the scopes 
                that are required for your application
            access_token (str): Defaults to None. Not used in this implementation.
            session_type (:class:`SessionBase<onedrivesdk.session_base.SessionBase>`):
                Defaults to :class:`Session<onedrivesdk.session.Session>`,
                the implementation of SessionBase that stores your
                session. WARNING: THE DEFAULT IMPLEMENTATION ONLY
                STORES SESSIONS TO THE RUN DIRECTORY IN PLAIN TEXT.
                THIS IS UNSAFE. IT IS HIGHLY RECOMMENDED THAT YOU
                IMPLEMENT YOUR OWN VERSION.
            loop (BaseEventLoop): Defaults to None, the asyncio
                loop to use for all async requests. If none is provided,
                asyncio.get_event_loop() will be called. If using Python
                3.3 or below this does not need to be specified
            auth_server_url (str): URL where OAuth authentication can be performed. If
                None, defaults to OAuth for Microsoft Account.
            auth_token_url (str): URL where OAuth token can be redeemed. If None,
                defaults to OAuth for Microsoft Account.
        """
        self._http_provider = http_provider
        self._client_id = client_id
        self._scopes = scopes
        self._session_type = Session if session_type is None else session_type
        self._session = None
        self._auth_server_url = self.MSA_AUTH_SERVER_URL if auth_server_url is None else auth_server_url
        self._auth_token_url = self.MSA_AUTH_TOKEN_URL if auth_token_url is None else auth_token_url

        if sys.version_info >= (3, 4, 0):
            import asyncio
            self._loop = loop if loop else asyncio.get_event_loop()

    @property
    def client_id(self):
        """Gets and sets the client_id for the
        AuthProvider

        Returns:
            str: The client id
        """
        return self._client_id

    @client_id.setter
    def client_id(self, value):
        self._client_id = value

    @property
    def scopes(self):
        """Gets and sets the scopes for the
        AuthProvider

        Returns:
            list of str: The scopes
        """
        return self._scopes

    @scopes.setter
    def scopes(self, value):
        self._scopes = value

    @property
    def access_token(self):
        """Gets and sets the access_token for the
            :class:`AuthProvider`

        Returns:
            str: The access token. Looks at the session to figure out what the access token is, since this
                class does not directly store the access token.
        """
        if self._session is not None:
            return self._session.access_token
        return None

    @property
    def auth_server_url(self):
        """Gets and sets the authorization server url for the
            :class:`AuthProvider`

        Returns:
            str: Auth server url
        """
        return self._auth_server_url

    @auth_server_url.setter
    def auth_server_url(self, value):
        self._auth_server_url = value

    @property
    def auth_token_url(self):
        """Gets and sets the authorization token url for the
        AuthProvider

        Returns:
            str: The auth token url
        """
        return self._auth_token_url

    @auth_token_url.setter
    def auth_token_url(self, value):
        self._auth_token_url = value

    def get_auth_url(self, redirect_uri, response_type=None):
        """Build the auth url using the params provided
        and the auth_provider

        Args:
            redirect_uri (str): The URI to redirect the response
                to
            response_type (str): Response type query param value.
                If not provided, defaults to 'code'. Should be either
                'code' or 'token'.
        """

        params = {
            "client_id": self.client_id,
            "response_type": "code" if response_type is None else response_type,
            "redirect_uri": redirect_uri
            }
        if self.scopes is not None:
            params["scope"] = " ".join(self.scopes)

        return "{}?{}".format(self._auth_server_url, urlencode(params))

    def authenticate(self, code, redirect_uri, client_secret, resource=None):
        """Takes in a code, gets the access token and creates a session.

        Args:
            code (str):
                The code provided by the oauth provider.
            redirect_uri (str): The URI to redirect the callback
                to
            client_secret (str): The client secret of your app.
            resource (str): Defaults to None,The resource  
                you want to access
        """
        params = {
            "client_id": self.client_id,
            "redirect_uri": redirect_uri,
            "client_secret": client_secret,
            "code": code,
            "response_type": "code",
            "grant_type": "authorization_code"
        }

        if resource is not None:
            params["resource"] = resource

        auth_url = self._auth_token_url
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        response = self._http_provider.send(method="POST",
                                            headers=headers,
                                            url=auth_url,
                                            data=params)

        rcont = json.loads(response.content)
        self._session = self._session_type(rcont["token_type"],
                                rcont["expires_in"],
                                rcont["scope"],
                                rcont["access_token"],
                                self.client_id,
                                self._auth_token_url,
                                redirect_uri,
                                rcont["refresh_token"] if "refresh_token" in rcont else None,
                                client_secret)

    def authenticate_request(self, request):
        """Append the required authentication headers
        to the specified request. This will only function
        if a session has been successfully created using
        :func:`authenticate`. This will also refresh the
        authentication token if necessary.

        Args:
            request (:class:`RequestBase<onedrivesdk.request_base.RequestBase>`):
                The request to authenticate
        """
        if self._session is None:
            raise RuntimeError("""Session must be authenticated 
                before applying authentication to a request.""")

        if self._session.is_expired() and 'wl.offline_access' in self.scopes:
            self.refresh_token()

        request.append_option(
            HeaderOption("Authorization",
                         "bearer {}".format(self._session.access_token)))

    def refresh_token(self):
        """Refresh the token currently used by the session"""
        if self._session is None:
            raise RuntimeError("""Session must be authenticated 
                before refreshing token.""")

        if self._session.refresh_token is None:
            raise RuntimeError("""Refresh token not present.""")

        params = {
            "refresh_token": self._session.refresh_token,
            "client_id": self._session.client_id,
            "redirect_uri": self._session.redirect_uri,
            "grant_type": "refresh_token"
        }

        if self._session.client_secret is not None:
            params["client_secret"] = self._session.client_secret

        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        response = self._http_provider.send(method="POST",
                                            headers=headers,
                                            url=self._session.auth_server_url,
                                            data=params)
        rcont = json.loads(response.content)
        self._session.refresh_session(rcont["expires_in"],
                                      rcont["scope"],
                                      rcont["access_token"],
                                      rcont["refresh_token"])

    def redeem_refresh_token(self, resource):
        """Redeem a refresh token against a new resource. Used
        only by OneDrive for Business apps.

        Args:
            resource (str): URL to resource to be accessed.
                Can be a 'serviceResourceId' value obtained from
                Discovery Service."""
        if self._session is None:
            raise RuntimeError("""Session must be authenticated
                before refreshing token.""")

        if self._session.refresh_token is None:
            raise RuntimeError("""Refresh token not present.""")

        params = {
            "client_id": self._session.client_id,
            "redirect_uri": self._session.redirect_uri,
            "client_secret": self._session.client_secret,
            "refresh_token": self._session.refresh_token,
            "grant_type": "refresh_token",
            "resource": resource
        }

        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        response = self._http_provider.send(method="POST",
                                            headers=headers,
                                            url=self.auth_token_url,
                                            data=params)
        rcont = json.loads(response.content)
        self._session.refresh_session(rcont["expires_in"],
                                      "",
                                      rcont["access_token"],
                                      rcont["refresh_token"])

    def save_session(self, **save_session_kwargs):
        """Save the current session. Must have already
        obtained an access_token.
        
        Args:
            save_session_kwargs (dict): Arguments to 
                be passed to save_session.
        """
        if self._session is None:
            raise RuntimeError("""Session must be authenticated before
            it can be saved. """)
        self._session.save_session(**save_session_kwargs)

    def load_session(self, **load_session_kwargs):
        """Load session. This will overwrite the current session.

        Args:
            load_session_kwargs (dict): Arguments to
                be passed to load_session.
        """
        self._session = self._session_type.load_session(**load_session_kwargs)



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.identity import Identity
from ..one_drive_object_base import OneDriveObjectBase


class IdentitySet(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def application(self):
        """
        Gets and sets the application
        
        Returns: 
            :class:`Identity<onedrivesdk.model.identity.Identity>`:
                The application
        """
        if "application" in self._prop_dict:
            if isinstance(self._prop_dict["application"], OneDriveObjectBase):
                return self._prop_dict["application"]
            else :
                self._prop_dict["application"] = Identity(self._prop_dict["application"])
                return self._prop_dict["application"]

        return None

    @application.setter
    def application(self, val):
        self._prop_dict["application"] = val
    @property
    def device(self):
        """
        Gets and sets the device
        
        Returns: 
            :class:`Identity<onedrivesdk.model.identity.Identity>`:
                The device
        """
        if "device" in self._prop_dict:
            if isinstance(self._prop_dict["device"], OneDriveObjectBase):
                return self._prop_dict["device"]
            else :
                self._prop_dict["device"] = Identity(self._prop_dict["device"])
                return self._prop_dict["device"]

        return None

    @device.setter
    def device(self, val):
        self._prop_dict["device"] = val
    @property
    def user(self):
        """
        Gets and sets the user
        
        Returns: 
            :class:`Identity<onedrivesdk.model.identity.Identity>`:
                The user
        """
        if "user" in self._prop_dict:
            if isinstance(self._prop_dict["user"], OneDriveObjectBase):
                return self._prop_dict["user"]
            else :
                self._prop_dict["user"] = Identity(self._prop_dict["user"])
                return self._prop_dict["user"]

        return None

    @user.setter
    def user(self, val):
        self._prop_dict["user"] = val



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.identity import Identity
from ..one_drive_object_base import OneDriveObjectBase


class SharingLink(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def application(self):
        """
        Gets and sets the application
        
        Returns: 
            :class:`Identity<onedrivesdk.model.identity.Identity>`:
                The application
        """
        if "application" in self._prop_dict:
            if isinstance(self._prop_dict["application"], OneDriveObjectBase):
                return self._prop_dict["application"]
            else :
                self._prop_dict["application"] = Identity(self._prop_dict["application"])
                return self._prop_dict["application"]

        return None

    @application.setter
    def application(self, val):
        self._prop_dict["application"] = val
    @property
    def type(self):
        """Gets and sets the type
        
        Returns: 
            str:
                The type
        """
        if "type" in self._prop_dict:
            return self._prop_dict["type"]
        else:
            return None

    @type.setter
    def type(self, val):
        self._prop_dict["type"] = val

    @property
    def web_url(self):
        """Gets and sets the webUrl
        
        Returns: 
            str:
                The webUrl
        """
        if "webUrl" in self._prop_dict:
            return self._prop_dict["webUrl"]
        else:
            return None

    @web_url.setter
    def web_url(self, val):
        self._prop_dict["webUrl"] = val

    @property
    def web_html(self):
        """Gets and sets the webHtml
        
        Returns: 
            str:
                The webHtml
        """
        if "webHtml" in self._prop_dict:
            return self._prop_dict["webHtml"]
        else:
            return None

    @web_html.setter
    def web_html(self, val):
        self._prop_dict["webHtml"] = val

    @property
    def configurator_url(self):
        """Gets and sets the configuratorUrl
        
        Returns: 
            str:
                The configuratorUrl
        """
        if "configuratorUrl" in self._prop_dict:
            return self._prop_dict["configuratorUrl"]
        else:
            return None

    @configurator_url.setter
    def configurator_url(self, val):
        self._prop_dict["configuratorUrl"] = val




'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
from __future__ import unicode_literals, with_statement
import requests
from .http_provider_base import HttpProviderBase
from .http_response import HttpResponse


class HttpProvider(HttpProviderBase):

    def send(self, method, headers, url, data=None, content=None, path=None):
        """Send the built request using all the specified
        parameters.

        Args:
            method (str): The HTTP method to use (ex. GET)
            headers (dict of (str, str)): A dictionary of name-value
                pairs for headers in the request
            url (str): The URL for the request to be sent to
            data (str): Defaults to None, data to include in the body
                of the request which is not in JSON format
            content (dict): Defaults to None, a dictionary to include
                in JSON format in the body of the request
            path (str): Defaults to None, the path to the local file
                to send in the body of the request

        Returns:
            :class:`HttpResponse<onedrivesdk.http_response.HttpResponse>`:
                The response to the request
        """
        session = requests.Session()

        if path:
            with open(path, mode='rb') as f:
                request = requests.Request(method,
                                           url,
                                           headers=headers,
                                           data=f)
                prepped = request.prepare()
                response = session.send(prepped)
        else:
            request = requests.Request(method,
                                       url,
                                       headers=headers,
                                       data=data,
                                       json=content)
            prepped = request.prepare()
            response = session.send(prepped)

        custom_response = HttpResponse(response.status_code, response.headers, response.text)
        return custom_response

    def download(self, headers, url, path):
        """Downloads a file to the stated path

        Args:
            headers (dict of (str, str)): A dictionary of name-value
                pairs to be used as headers in the request
            url (str): The URL from which to download the file
            path (str): The local path to save the downloaded file

        Returns:
            :class:`HttpResponse<onedrivesdk.http_response.HttpResponse>`:
                The response to the request
        """
        response = requests.get(
            url,
            stream=True,
            headers=headers)

        if response.status_code == 200:
            with open(path, 'wb') as f:
                for chunk in response.iter_content(chunk_size=1024):
                    if chunk:
                        f.write(chunk)
                        f.flush()
            custom_response = HttpResponse(response.status_code, response.headers, None)
        else:
            custom_response = HttpResponse(response.status_code, response.headers, response.text)

        return custom_response



from __future__ import with_statement
from setuptools import setup
from codecs import open
from os import path
import sys

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'DESCRIPTION.rst'), encoding='utf-8') as f:
    long_description = f.read()

with open(path.join(here, 'src/onedrivesdk/version.txt'), encoding='utf-8') as f:
    version = f.read()


def main():
    package_list = ['onedrivesdk',
                    'onedrivesdk.request',
                    'onedrivesdk.model',
                    'onedrivesdk.extensions',
                    'onedrivesdk.helpers']

    if sys.version_info >= (3, 4):
        base_dir = 'python3'
        package_list.append('onedrivesdk.version_bridge')
    else:
        base_dir = 'python2'

    setup(
        name='onedrivesdk',

        version=version,

        description='Official Python OneDrive SDK for interfacing with OneDrive APIs',
        long_description=long_description,

        url='http://dev.onedrive.com',

        author='Microsoft',
        author_email='',

        license='MIT',

        classifiers=[
            'Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3.4',
            'Operating System :: OS Independent',
            'Operating System :: POSIX',
            'Operating System :: Microsoft :: Windows',
            'Operating System :: MacOS :: MacOS X',
        ],

        keywords='onedrive sdk microsoft',

        packages=package_list,

        package_dir={'onedrivesdk': 'src/onedrivesdk',
                     'onedrivesdk.request': 'src/' + base_dir + '/request'},

        package_data={'onedrivesdk': ['version.txt']},

        install_requires=['requests>=2.6.1'],

        extras_require={
            "samples": ["Pillow"],
            "tests": ["Mock"]
        },

        test_suite='testonedrivesdk'
    )

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..model.identity_set import IdentitySet
from ..model.sharing_invitation import SharingInvitation
from ..model.item_reference import ItemReference
from ..model.sharing_link import SharingLink
from ..one_drive_object_base import OneDriveObjectBase


class Permission(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def granted_to(self):
        """
        Gets and sets the grantedTo
        
        Returns: 
            :class:`IdentitySet<onedrivesdk.model.identity_set.IdentitySet>`:
                The grantedTo
        """
        if "grantedTo" in self._prop_dict:
            if isinstance(self._prop_dict["grantedTo"], OneDriveObjectBase):
                return self._prop_dict["grantedTo"]
            else :
                self._prop_dict["grantedTo"] = IdentitySet(self._prop_dict["grantedTo"])
                return self._prop_dict["grantedTo"]

        return None

    @granted_to.setter
    def granted_to(self, val):
        self._prop_dict["grantedTo"] = val

    @property
    def id(self):
        """
        Gets and sets the id
        
        Returns:
            str:
                The id
        """
        if "id" in self._prop_dict:
            return self._prop_dict["id"]
        else:
            return None

    @id.setter
    def id(self, val):
        self._prop_dict["id"] = val

    @property
    def invitation(self):
        """
        Gets and sets the invitation
        
        Returns: 
            :class:`SharingInvitation<onedrivesdk.model.sharing_invitation.SharingInvitation>`:
                The invitation
        """
        if "invitation" in self._prop_dict:
            if isinstance(self._prop_dict["invitation"], OneDriveObjectBase):
                return self._prop_dict["invitation"]
            else :
                self._prop_dict["invitation"] = SharingInvitation(self._prop_dict["invitation"])
                return self._prop_dict["invitation"]

        return None

    @invitation.setter
    def invitation(self, val):
        self._prop_dict["invitation"] = val

    @property
    def inherited_from(self):
        """
        Gets and sets the inheritedFrom
        
        Returns: 
            :class:`ItemReference<onedrivesdk.model.item_reference.ItemReference>`:
                The inheritedFrom
        """
        if "inheritedFrom" in self._prop_dict:
            if isinstance(self._prop_dict["inheritedFrom"], OneDriveObjectBase):
                return self._prop_dict["inheritedFrom"]
            else :
                self._prop_dict["inheritedFrom"] = ItemReference(self._prop_dict["inheritedFrom"])
                return self._prop_dict["inheritedFrom"]

        return None

    @inherited_from.setter
    def inherited_from(self, val):
        self._prop_dict["inheritedFrom"] = val

    @property
    def link(self):
        """
        Gets and sets the link
        
        Returns: 
            :class:`SharingLink<onedrivesdk.model.sharing_link.SharingLink>`:
                The link
        """
        if "link" in self._prop_dict:
            if isinstance(self._prop_dict["link"], OneDriveObjectBase):
                return self._prop_dict["link"]
            else :
                self._prop_dict["link"] = SharingLink(self._prop_dict["link"])
                return self._prop_dict["link"]

        return None

    @link.setter
    def link(self, val):
        self._prop_dict["link"] = val

    @property
    def roles(self):
        """
        Gets and sets the roles
        
        Returns:
            str:
                The roles
        """
        if "roles" in self._prop_dict:
            return self._prop_dict["roles"]
        else:
            return None

    @roles.setter
    def roles(self, val):
        self._prop_dict["roles"] = val

    @property
    def share_id(self):
        """
        Gets and sets the shareId
        
        Returns:
            str:
                The shareId
        """
        if "shareId" in self._prop_dict:
            return self._prop_dict["shareId"]
        else:
            return None

    @share_id.setter
    def share_id(self, val):
        self._prop_dict["shareId"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from datetime import datetime
from ..one_drive_object_base import OneDriveObjectBase


class FileSystemInfo(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def created_date_time(self):
        """Gets and sets the createdDateTime
        
        Returns: 
            datetime:
                The createdDateTime
        """
        if "createdDateTime" in self._prop_dict:
            if '.' in self._prop_dict["createdDateTime"]:
                return datetime.strptime(self._prop_dict["createdDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["createdDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @created_date_time.setter
    def created_date_time(self, val):
        self._prop_dict["createdDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")

    @property
    def last_modified_date_time(self):
        """Gets and sets the lastModifiedDateTime
        
        Returns: 
            datetime:
                The lastModifiedDateTime
        """
        if "lastModifiedDateTime" in self._prop_dict:
            if '.' in self._prop_dict["lastModifiedDateTime"]:
                return datetime.strptime(self._prop_dict["lastModifiedDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S.%f")
            else:
                return datetime.strptime(self._prop_dict["lastModifiedDateTime"].replace("Z", ""), "%Y-%m-%dT%H:%M:%S")
        else:
            return None

    @last_modified_date_time.setter
    def last_modified_date_time(self, val):
        self._prop_dict["lastModifiedDateTime"] = val.isoformat()+((".0" if val.time().microsecond == 0 else "")+"Z")




'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
import asyncio
from ..auth_provider import AuthProvider


@asyncio.coroutine
def authenticate_async(self, code, redirect_uri, client_secret=None):
    """Takes in a code string, gets the access token,
    and creates session property bag in async

    Args:
        code (str):
            The code provided by the oauth provider
        redirect_uri (str): The URI to redirect the callback
            to
        client_secret (str): Defaults to None, the client
            secret of your app
    """
    future = self._loop.run_in_executor(None,
                                        self.authenticate,
                                        code,
                                        redirect_uri,
                                        client_secret)
    yield from future


@asyncio.coroutine
def authenticate_request_async(self, request):
    """Authenticate and append the required
    headers to the request in async

    Args:
        request (:class:`RequestBase<onedrivesdk.request_base.RequestBase>`):
            The request to authenticate
    """
    future = self._loop.run_in_executor(None,
                                        self.authenticate_request,
                                        request)
    yield from future


@asyncio.coroutine
def refresh_token_async(self):
    """Refresh the token currently used by the session"""
    future = self._loop.run_in_executor(None,
                                        self.refresh_token)
    yield from future

AuthProvider.authenticate_async = authenticate_async
AuthProvider.authenticate_request_async = authenticate_async
AuthProvider.refresh_token_async = authenticate_async



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from .subscription_request import SubscriptionRequest
from ..request_builder_base import RequestBuilderBase


class SubscriptionRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client):
        """Initialize the SubscriptionRequestBuilder

        Args:
            request_url (str): The url to perform the SubscriptionRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
        """
        super(SubscriptionRequestBuilder, self).__init__(request_url, client)

    def request(self, expand=None, select=None, options=None):
        """Builds the SubscriptionRequest

        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request. Defaults to None.

        Returns:
            :class:`SubscriptionRequest<onedrivesdk.request.subscription_request.SubscriptionRequest>`:
                The SubscriptionRequest
        """
        req = SubscriptionRequest(self._request_url, self._client, options)
        req._set_query_options(expand=expand, select=select)
        return req

    def delete(self):
        """Deletes the specified Subscription."""
        self.request().delete()

    def get(self):
        """Gets the specified Subscription.
        
        Returns:
            :class:`Subscription<onedrivesdk.model.subscription.Subscription>`:
                The Subscription.
        """
        return self.request().get()

    def update(self, subscription):
        """Updates the specified Subscription.
        
        Args:
            subscription (:class:`Subscription<onedrivesdk.model.subscription.Subscription>`):
                The Subscription to update.

        Returns:
            :class:`Subscription<onedrivesdk.model.subscription.Subscription>`:
                The updated Subscription
        """
        return self.request().update(subscription)




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from ..collection_base import CollectionRequestBase
from ..request_builder_base import RequestBuilderBase
from ..options import *
import json


class ItemDeltaRequest(CollectionRequestBase):

    def __init__(self, request_url, client, options, token=None):
        super(ItemDeltaRequest, self).__init__(request_url, client, options)
        self.method = "GET"

        if token:
            self._query_options["token"] = token

    def get(self):
        """Sends the GET request
        
        Returns:
            :class:`ItemDeltaCollectionResponse<onedrivesdk.request.item_delta_collection.ItemDeltaCollectionResponse>`:
                The resulting collection page from the operation
        """
        collection_response = ItemDeltaCollectionResponse(json.loads(self.send().content))
        return self._page_from_response(collection_response)

    
    @staticmethod
    def get_next_page_request(collection_page, client, options, token=None):
        """Gets the ItemDeltaRequest for the next page. Returns None if there is no next page

        Yields: 
            :class:`ItemDeltaRequest<onedrivesdk.request.item_delta.ItemDeltaRequest>`:
                The ItemDeltaRequest
        """
        if collection_page._next_page_link:
            return ItemDeltaRequest(collection_page._next_page_link, client, options, token)
        else:
            return None


class ItemDeltaRequestBuilder(RequestBuilderBase):

    def __init__(self, request_url, client, token=None):
        super(ItemDeltaRequestBuilder, self).__init__(request_url, client)
        self._method_options = {}

        self._method_options["token"] = token

    def request(self, expand=None, select=None, top=None, order_by=None, options=None):
        """Builds the request for the ItemDelta
        
        Args:
            expand (str): Default None, comma-seperated list of relationships
                to expand in the response.
            select (str): Default None, comma-seperated list of properties to
                include in the response.
            top (int): Default None, the number of items to return in a result.
            order_by (str): Default None, comma-seperated list of properties
                that are used to sort the order of items in the response.
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                Default to None, list of options to include in the request

        Returns: 
            :class:`ItemDeltaRequest<onedrivesdk.request.item_delta.ItemDeltaRequest>`:
                The request
        """
        req = ItemDeltaRequest(self._request_url, self._client, options, token=self._method_options["token"])
        req._set_query_options(expand=expand, select=select, top=top, order_by=order_by)
        return req

    def get(self):
        """Sends the GET request
        
        Returns:
            :class:`ItemDeltaCollectionResponse<onedrivesdk.request.item_delta_collection.ItemDeltaCollectionResponse>`:
            The resulting ItemDeltaCollectionResponse from the operation
        """
        return self.request().get()

from ..request.item_delta_collection import ItemDeltaCollectionResponse



'''
------------------------------------------------------------------------------
 Copyright (c) 2015 Microsoft Corporation

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
------------------------------------------------------------------------------
'''
from __future__ import unicode_literals
from .session_base import SessionBase
from time import time


class Session(SessionBase):

    def __init__(self,
                 token_type,
                 expires_in,
                 scope_string,
                 access_token,
                 client_id,
                 auth_server_url,
                 redirect_uri,
                 refresh_token=None,
                 client_secret=None):
        self.token_type = token_type
        self._expires_at = time() + int(expires_in)
        self.scope = scope_string.split(" ")
        self.access_token = access_token
        self.client_id = client_id
        self.auth_server_url = auth_server_url
        self.redirect_uri = redirect_uri
        self.refresh_token = refresh_token
        self.client_secret = client_secret

    def is_expired(self):
        """Whether or not the session has expired

        Returns:
            bool: True if the session has expired, otherwise false
        """
        # Add a 10 second buffer in case the token is just about to expire
        return self._expires_at < time() - 10

    def refresh_session(self, expires_in, scope_string, access_token, refresh_token):
        self._expires_at = time() + int(expires_in)
        self.scope = scope_string.split(" ")
        self.access_token = access_token
        self.refresh_token = refresh_token

    def save_session(self, **save_session_kwargs):
        """Save the current session.
        IMPORTANT: This implementation should only be used for debugging.
        For real applications, the Session object should be subclassed and
        both save_session() and load_session() should be overwritten using
        the client system's correct mechanism (keychain, database, etc.).
        Remember, the access_token should be treated the same as a password.
        
        Args:
            save_session_kwargs (dicr): To be used by implementation
            of save_session, however save_session wants to use them. The
            default implementation (this one) takes a relative or absolute
            file path for pickle save location, under the name "path"
        """
        path = "session.pickle"
        if "path" in save_session_kwargs:
            path = save_session_kwargs["path"]
        
        with open(path, "wb") as session_file:
            import pickle
            # pickle.HIGHEST_PROTOCOL is binary format. Good perf.
            pickle.dump(self, session_file, pickle.HIGHEST_PROTOCOL)

    @staticmethod
    def load_session(**load_session_kwargs):
        """Save the current session.
        IMPORTANT: This implementation should only be used for debugging.
        For real applications, the Session object should be subclassed and
        both save_session() and load_session() should be overwritten using
        the client system's correct mechanism (keychain, database, etc.).
        Remember, the access_token should be treated the same as a password.
        
        Args:
            load_session_kwargs (dict): To be used by implementation
            of load_session, however load_session wants to use them. The
            default implementation (this one) takes a relative or absolute
            file path for pickle save location, under the name "path"

        Returns:
            :class:`Session`: The loaded session
        """
        path = "session.pickle"
        if "path" in load_session_kwargs:
            path = load_session_kwargs["path"]
        
        with open(path, "rb") as session_file:
            import pickle
            return pickle.load(session_file)



# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from __future__ import unicode_literals
from ..one_drive_object_base import OneDriveObjectBase


class Location(OneDriveObjectBase):

    def __init__(self, prop_dict=None):
        self._prop_dict = prop_dict if prop_dict is not None else {}

    @property
    def altitude(self):
        """Gets and sets the altitude
        
        Returns: 
            float:
                The altitude
        """
        if "altitude" in self._prop_dict:
            return self._prop_dict["altitude"]
        else:
            return None

    @altitude.setter
    def altitude(self, val):
        self._prop_dict["altitude"] = val

    @property
    def latitude(self):
        """Gets and sets the latitude
        
        Returns: 
            float:
                The latitude
        """
        if "latitude" in self._prop_dict:
            return self._prop_dict["latitude"]
        else:
            return None

    @latitude.setter
    def latitude(self, val):
        self._prop_dict["latitude"] = val

    @property
    def longitude(self):
        """Gets and sets the longitude
        
        Returns: 
            float:
                The longitude
        """
        if "longitude" in self._prop_dict:
            return self._prop_dict["longitude"]
        else:
            return None

    @longitude.setter
    def longitude(self, val):
        self._prop_dict["longitude"] = val




# -*- coding: utf-8 -*- 
'''
# Copyright (c) Microsoft Corporation.  All Rights Reserved.  Licensed under the MIT License.  See License in the project root for license information.
# 
#  This file was generated and any changes will be overwritten.
'''

from ..request_base import RequestBase
from ..model.share import Share
import json

class ShareRequest(RequestBase):
    """The type ShareRequest."""
    
    def __init__(self, request_url, client, options):
        """Constructs a new ShareRequest.

        Args:
            request_url (str): The url to perform the ShareRequest
                on
            client (:class:`OneDriveClient<onedrivesdk.request.one_drive_client.OneDriveClient>`):
                The client which will be used for the request
            options (list of :class:`Option<onedrivesdk.options.Option>`):
                A list of options to pass into the request
        """
        super(ShareRequest, self).__init__(request_url, client, options)

    def delete(self):
        """Deletes the specified Share."""
        self.method = "DELETE"
        self.send()

    def get(self):
        """Gets the specified Share.
        
        Returns:
            :class:`Share<onedrivesdk.model.share.Share>`:
                The Share.
        """
        self.method = "GET"
        entity = Share(json.loads(self.send().content))
        self._initialize_collection_properties(entity)
        return entity

    def update(self, share):
        """Updates the specified Share.
        
        Args:
            share (:class:`Share<onedrivesdk.model.share.Share>`):
                The Share to update.

        Returns:
            :class:`Share<onedrivesdk.model.share.Share>`:
                The updated Share.
        """
        self.content_type = "application/json"
        self.method = "PATCH"
        entity = Share(json.loads(self.send(share).content))
        self._initialize_collection_properties(entity)
        return entity

    def _initialize_collection_properties(self, value):
        if value and value._prop_dict:
            if value.items:
                if "items@odata.nextLink" in value._prop_dict:
                    next_page_link = value._prop_dict["items@odata.nextLink"]
                    value.items._next_page_link = next_page_link
